package speiger.src.coreengine.rendering.gui.font;

import java.util.Map;

import speiger.src.collections.ints.maps.impl.hash.Int2ObjectOpenHashMap;
import speiger.src.collections.ints.maps.interfaces.Int2ObjectMap;
import speiger.src.collections.objects.maps.impl.hash.Object2ObjectOpenHashMap;
import speiger.src.coreengine.assets.AssetLocation;
import speiger.src.coreengine.rendering.gui.font.glyth.Glyth;
import speiger.src.coreengine.rendering.gui.font.glyth.GlythData;

public class GlythCache {
	private Font font;
	final int style;
	Map<AssetLocation, FontCache> cache = new Object2ObjectOpenHashMap<>();
	
	public GlythCache(Font font, int style) {
		this.font = font;
		this.style = style;
	}
	
	private FontCache cache(AssetLocation font) {
		return cache.computeIfAbsent(font, FontCache::new);
	}
	
	public GlythData data(AssetLocation fontId, int codepoint) {
		return cache(fontId).data(codepoint);
	}
	
	public Glyth glyth(AssetLocation fontId, int codepoint) {
		return cache(fontId).glyth(codepoint);
	}
	
	public class FontCache {
		Int2ObjectMap<GlythData> data = new Int2ObjectOpenHashMap<>();
		Int2ObjectMap<Glyth> glyth = new Int2ObjectOpenHashMap<>();
		AssetLocation fontId;
		
		public FontCache(AssetLocation fontId) {
			this.fontId = fontId;
		}

		public GlythData data(int codepoint) {
			return data.computeIfAbsent(codepoint, this::compute);
		}
		
		public Glyth glyth(int codepoint) {
			return glyth.computeIfAbsent(codepoint, this::bake);
		}
		
		private GlythData compute(int codepoint) {
			return new GlythData(font.font(fontId).unbaked(codepoint, style), font.size);
		}
		
		private Glyth bake(int codepoint) {
			return font.font(fontId).unbaked(codepoint, style).bake(font.baker, font.size, font.oversample);
		}

	}
}